;-----------------------------------------------------------------------;
;	Replay STe (multiplexing)					;
;	Copyright (C) Karl Anders ygard, 15 December 1993		;
;-----------------------------------------------------------------------;
; Uses no registers							;
;									;
; This replay routine facilitates the multiplexing mode as suggested	;
; by Jarle Kloppbakken of Imagina.					;
;									;
; Replay is started by calling MUSON in supervisor mode. Returns with	;
; timer A running. Calling MUSOFF will stop music.			;
;									;
; This routine needs some workspace after the module to work properly.	;
; I've set it to 16KB, some modules may need, some may need less. If	;
; the workspace is too small, execution will stop on an ILLEGAL		;
; instruction. Adjust workspace size (WSPACE) to fit the specific	;
; module.								;
;									;
; FREQ = replay frequency (0=6.259kHz, 1=12.517kHz, 2=25.036kHz		;
;			   3=50.072kHz)					;
; WSPACE = workspace size in bytes					;
;-----------------------------------------------------------------------;

WSPACE	EQU 16384			; Workspace size in bytes
FREQ	EQU 3				; 0=6.259, 1=12.517, 2=25.036
					; 3=50.072

	IFEQ FREQ			; 6.259kHz
PARTS	EQU 5-1
LEN	EQU 25
INC	EQU $023BF313			; 3579546/6125*65536
	ELSEIF
	IFEQ FREQ-1			; 12.517kHz
PARTS	EQU 5-1
LEN	EQU 50
INC	EQU $023BF313			; 3579546/6125*65536
	ELSEIF
	IFEQ FREQ-2			; 25.035kHz
PARTS	EQU 5-1
LEN	EQU 50
INC	EQU $011DF989			; 3579546/12517*65536
	ELSEIF
	IFEQ FREQ-3			; 50.072kHz
PARTS	EQU 5-1
LEN	EQU 100
INC	EQU $008EFB4E			; 3579546/25035*65536
	ELSEIF
	FAIL
	END
	ENDC
	ENDC
	ENDC
	ENDC

;--------------------------8<----- Cut here -----8<---------------------;
;			Demo bit which starts replay.			;

	pea	muson(pc)
	move.w	#$0026,-(sp)
	trap	#14			; Supexec - start music
	addq.w	#6,sp

	move.w	#7,-(sp)
	trap	#1
	addq.w	#2,sp

	pea	musoff(pc)
	move.w	#$0026,-(sp)
	trap	#14			; Supexec - stop music
	addq.w	#6,sp

	pea	$004C0000
	trap	#1			; Terminate

;--------------------------8<----- Cut here -----8<---------------------;

;---------------------------------------------------- Interrupts on/off --
muson	bsr	vol			; Calculate volume tables
	bsr	incrcal			; Calculate tonetables

	bsr	mt_init			; Initialize music

	cmp.l	#workspc-mt_data,d1
	ble.s	memok

	illegal

memok	bsr	mt_prepare		; Prepare samples

	move	#$2700,sr

	bset	#5,$FFFFFA07.w
	bset	#5,$FFFFFA13.w

	clr.b	$FFFFFA19.w
	move.b	#1,$FFFFFA1F.w
	move.b	#8,$FFFFFA19.w

	move.l	$00000134.w,oldtima
	move.l	#stereo,$00000134.w

	move.b	#FREQ,$FFFF8921.w	; Frequency

	lea	$FFFF8907.w,a0

	move.l	#sample1,d0
	move.b	d0,(a0)
	lsr.w	#8,d0
	move.l	d0,-5(a0)

	move.l	#sample1+LEN*2,d0
	move.b	d0,12(a0)
	lsr.w	#8,d0
	move.l	d0,7(a0)

	move.b	#3,$FFFF8901.w		; Start DMA

	move	#$2300,sr
	rts

musoff	move	#$2700,sr

	clr.b	$FFFFFA19.w		; Stop timers

	move.l	oldtima(pc),$00000134.w ; Restore everything

	bclr	#5,$FFFFFA07.w
	bclr	#5,$FFFFFA13.w

	clr.b	$FFFF8901.w		; Stop DMA

	move	#$2300,sr
	rts

oldtima	DC.L 0

;--------------------------------------------------------- Volume table --
vol	moveq	#64,d0
	lea	vtabend(pc),a0

.ploop	move.w	#255,d1
.mloop	move.w	d1,d2
	ext.w	d2
	muls	d0,d2
	divs	#$40,d2			; <---- Master volume
	move.b	d2,-(a0)
	dbra	d1,.mloop
	dbra	d0,.ploop

	rts

vtab	DS.B 65*256
vtabend:

;------------------------------------------------------ Increment-table --
incrcal	lea	stab(pc),a0
	moveq	#$0030,d1
	move.w	#$039F-$0030,d0
	move.l	#INC,d2

recalc	swap	d2
	moveq	#0,d3
	move.w	d2,d3
	divu	d1,d3
	move.w	d3,d4
	swap	d4

	swap	d2
	move.w	d2,d3
	divu	d1,d3
	move.w	d3,d4
	move.l	d4,(a0)+

	addq.w	#1,d1
	dbra	d0,recalc
	rts

itab	DS.L $0030
stab	DS.L $03A0-$0030

;-------------------------------------------------------- DMA interrupt --
stereo	move	#$2500,sr
	bclr	#5,$FFFFFA0F.w
	movem.l d0-a6,-(sp)

	move.l	samp1(pc),d0
	move.l	samp2(pc),samp1
	move.l	d0,samp2

	lea	$FFFF8907.w,a0
	move.l	samp2(pc),d0
	move.b	d0,(a0)
	lsr.w	#8,d0
	move.l	d0,-5(a0)

	move.l	samp2(pc),d0
	IFGT FREQ-1
	add.l	#LEN*2*2,d0
	ELSE
	add.l	#LEN*2,d0
	ENDC
	move.b	d0,12(a0)
	lsr.w	#8,d0
	move.l	d0,7(a0)

	subq.w	#1,count
	bpl.s	.nomus

	move.w	#PARTS,count
	bsr	mt_music

.nomus	lea	itab(pc),a5
	lea	vtab(pc),a3
	moveq	#0,d4

	movea.l rep4lc(pc),a1
	move.w	rep4frc(pc),d0
	swap	d0

	move.w	aud4per(pc),d7
	add.w	d7,d7
	add.w	d7,d7
	move.l	0(a5,d7.w),d2
	swap	d2
	move.w	d2,d6

	move.w	aud4vol(pc),d7
	asl.w	#8,d7
	lea	0(a3,d7.w),a2


	movea.l rep1lc(pc),a0
	move.w	rep1frc(pc),d1

	move.w	aud1per(pc),d7
	add.w	d7,d7
	add.w	d7,d7
	move.w	0(a5,d7.w),d2
	movea.w 2(a5,d7.w),a5

	move.w	aud1vol(pc),d7
	asl.w	#8,d7
	lea	0(a3,d7.w),a3

	movea.l samp2(pc),a6
	addq.w	#1,a6
	moveq	#0,d3
	clr.w	d0

	bsr	calc

	IFGT FREQ-1
	adda.w	d0,a0
	adda.l	d4,a1
	clr.w	d0
	moveq	#0,d4

	cmpa.l	rep1end(pc),a0
	blt.s	.ok1_1
	suba.l	rep1rpt(pc),a0
.ok1_1	cmpa.l	rep4end(pc),a1
	blt.s	.ok4_1
	suba.l	rep4rpt(pc),a1

.ok4_1	bsr	calc
	ENDC

	adda.w	d0,a0
	adda.l	d4,a1

	cmpa.l	rep1end(pc),a0
	blt.s	.ok1_2
	suba.l	rep1rpt(pc),a0
.ok1_2	move.l	a0,rep1lc
	move.w	d1,rep1frc

	cmpa.l	rep4end(pc),a1
	blt.s	.ok4_2
	suba.l	rep4rpt(pc),a1
.ok4_2	move.l	a1,rep4lc
	swap	d0
	move.w	d0,rep4frc


	lea	itab(pc),a5
	lea	vtab(pc),a3
	moveq	#0,d4


	movea.l rep3lc(pc),a1
	move.w	rep3frc(pc),d0
	swap	d0

	move.w	aud3per(pc),d7
	add.w	d7,d7
	add.w	d7,d7
	move.l	0(a5,d7.w),d2
	swap	d2
	move.w	d2,d6

	move.w	aud3vol(pc),d7
	asl.w	#8,d7
	lea	0(a3,d7.w),a2


	movea.l rep2lc(pc),a0
	move.w	rep2frc(pc),d1

	move.w	aud2per(pc),d7
	add.w	d7,d7
	add.w	d7,d7
	move.w	0(a5,d7.w),d2
	movea.w 2(a5,d7.w),a5

	move.w	aud2vol(pc),d7
	asl.w	#8,d7
	lea	0(a3,d7.w),a3


	movea.l samp2(pc),a6
	moveq	#0,d3
	clr.w	d0

	bsr.s	calc

	IFGT FREQ-1
	adda.w	d0,a0
	adda.l	d4,a1
	clr.w	d0
	moveq	#0,d4

	cmpa.l	rep2end(pc),a0
	blt.s	.ok2_1
	suba.l	rep2rpt(pc),a0
.ok2_1	cmpa.l	rep3end(pc),a1
	blt.s	.ok3_1
	suba.l	rep3rpt(pc),a1

.ok3_1	bsr.s	calc
	ENDC

	adda.w	d0,a0
	adda.l	d4,a1
	cmpa.l	rep2end(pc),a0
	blt.s	.ok2_2
	suba.l	rep2rpt(pc),a0
.ok2_2	move.l	a0,rep2lc
	move.w	d1,rep2frc

	cmpa.l	rep3end(pc),a1
	blt.s	.ok3_2
	suba.l	rep3rpt(pc),a1
.ok3_2	move.l	a1,rep3lc
	swap	d0
	move.w	d0,rep3frc

	movem.l (sp)+,d0-a6
	rte

calc
	REPT LEN/2
	add.w	a5,d1
	addx.l	d2,d0
	addx.w	d6,d4

	move.b	0(a0,d0.w),d3
	move.b	0(a3,d3.w),(a6)

	move.b	0(a1,d4.w),d3
	move.b	0(a2,d3.w),2(a6)
	addq.w	#4,a6
	ENDR

	rts

;-------------------------------------------- Hardware-registers & data --
count	DC.W PARTS

rep1lc	DC.L sample1
rep1rpt	DC.L 0
rep1end	DC.L 0
rep1frc	DC.W 0
	DC.W 0

rep2lc	DC.L sample1
rep2rpt	DC.L 0
rep2end	DC.L 0
rep2frc	DC.W 0
	DC.W 0

rep3lc	DC.L sample1
rep3rpt	DC.L 0
rep3end	DC.L 0
rep3frc	DC.W 0
	DC.W 0

rep4lc	DC.L sample1
rep4rpt	DC.L 0
rep4end	DC.L 0
rep4frc	DC.W 0
	DC.W 0

aud1lc	DC.L dummy
aud1len	DC.W 0
aud1per	DC.W 0
aud1vol	DC.W 0
	DS.W 3

aud2lc	DC.L dummy
aud2len	DC.W 0
aud2per	DC.W 0
aud2vol	DC.W 0
	DS.W 3

aud3lc	DC.L dummy
aud3len	DC.W 0
aud3per	DC.W 0
aud3vol	DC.W 0
	DS.W 3

aud4lc	DC.L dummy
aud4len	DC.W 0
aud4per	DC.W 0
aud4vol	DC.W 0

dmactrl	DC.W 0

dummy	DC.L 0

samp1	DC.L sample1
samp2	DC.L sample2

	IFGT FREQ-1
sample1	DS.W LEN*2
sample2	DS.W LEN*2
	ELSE
sample1	DS.W LEN
sample2	DS.W LEN
	ENDC

	INCLUDE	replay.s

mt_data	INCBIN 'D:\HISCORE.MOD'

	DS.W WSPACE				; Workspace
workspc	DS.W 1
	